@@ -1,6 +1,6 @@
 /* callbacks.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -19,12 +19,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef _CALLBACKS_H
+#define _CALLBACKS_H
+
 #include <gnome.h>
 #include <glade/glade.h>
 #include "gui.h"
 
-void on_adjustment_trackvolumes_changed(GtkAdjustment *, gpointer);
-void on_togglebutton_trackvolumes_toggled(GtkToggleButton *, gpointer);
+void on_adjustment_trackvolumes_volume_changed(GtkAdjustment *, gpointer);
+void on_togglebutton_trackvolumes_mute_toggled(GtkToggleButton *, gpointer);
+void on_togglebutton_trackvolumes_solo_toggled(GtkToggleButton *, gpointer);
+void on_entry_trackvolumes_name_changed(GtkEditable *, gpointer);
 void on_button_block_clicked(GtkButton *, gpointer);
 gboolean keyboard_event(GtkWidget *, GdkEventKey *, gboolean, struct gui *);
 void on_selection_message_changed(GtkTreeSelection *, gpointer);
@@ -50,19 +55,22 @@ void on_cell_playseq_edited (GtkCellRendererText *, gchar *, gchar *, gpointer);
 void on_cell_blocklist_edited (GtkCellRendererText *, gchar *, gchar *, gpointer);
 void on_cell_messagelist_edited (GtkCellRendererText *, gchar *, gchar *, gpointer);
 void on_cell_messagelist_toggled (GtkCellRendererToggle *, gchar *, gpointer);
+void on_cell_preferences_midi_interfaces_toggled (GtkCellRendererToggle *, gchar *, gpointer);
 gboolean on_entry_instrumentproperties_name_changed(GtkEditable *, gpointer);
 void on_spinbutton_instrument_changed(GtkEditable *, gpointer);
 gboolean on_entry_songname_changed(GtkEditable *, gpointer);
 void on_checkbutton_sendsync_toggled(GtkToggleButton *, gpointer);
 void on_scale_tempo_value_changed(GtkRange *, gpointer);
 void on_scale_tpl_value_changed(GtkRange *, gpointer);
-gboolean on_entry_preferences_midi_device_changed(GtkEditable *, gpointer);
-void on_menuitem_preferences_rtc_activate(GtkMenuItem *, gpointer);
-void on_menuitem_preferences_nanosleep_activate(GtkMenuItem *, gpointer);
+gboolean on_entry_preferences_midi_interface_changed(GtkEditable *, gpointer);
 void on_scale_volume_value_changed(GtkRange *, gpointer);
 void on_scale_transpose_value_changed(GtkRange *, gpointer);
 void on_scale_hold_value_changed(GtkRange *, gpointer);
 void on_scale_midich_value_changed(GtkRange *, gpointer);
 void on_spinbutton_space_changed(GtkSpinButton *, gpointer);
 void on_checkbutton_edit_toggled(GtkToggleButton *, gpointer);
-void on_optionmenu_keyboard_changed(GtkOptionMenu *, gpointer);
+void on_combobox_keyboard_changed(GtkComboBox *, gpointer);
+void on_combobox_preferences_scheduling_mode_changed(GtkComboBox *, gpointer);
+void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox *, gpointer);
+
+#endif
