@@ -1,6 +1,6 @@
 /* midi.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -23,11 +23,59 @@
 #define _MIDI_H
 
 #include <libxml/tree.h>
+#include <alsa/asoundlib.h>
+
+#define MIDI_BUFFER_BLOCK_SIZE 4096
+
+enum {
+  MIDI_NULL,
+  MIDI_OSS,
+  MIDI_ALSA,
+  MIDI_BUFFER
+};
+
+enum {
+  DIRECTION_NONE = 0,
+  DIRECTION_IN = 1,
+  DIRECTION_OUT = 2,
+  DIRECTION_BOTH = 3
+};
 
 /* MIDI data structure */
 struct midi {
-  /* MIDI device file descriptor */
-  int fd;
+  /* Output device name */
+  char *devicename;
+  /* User readable name */
+  char *name;
+  /* Output device type */
+  unsigned int type;
+  /* Output device direction */
+  unsigned int direction;
+  /* Output device structure */
+  void *data;
+  /* Current tick and tick of last event */
+  unsigned int tick, oldtick;
+};
+
+struct midi_oss {
+  /* OSS input MIDI device file descriptor */
+  int in;
+  /* OSS output MIDI device file descriptor */
+  int out;
+};
+
+struct midi_alsa {
+  snd_rawmidi_t *in;
+  snd_rawmidi_t *out;
+};
+
+struct midi_buffer {
+  /* MIDI outbut buffer */
+  unsigned char *data;
+  /* Current position */
+  unsigned char *ptr;
+  /* Number of bytes currently allocated */
+  unsigned int allocated;
 };
 
 struct message {
@@ -41,10 +89,22 @@ struct message {
   unsigned int autosend;
 };
 
-/* Opens a MIDI device */
-struct midi *midi_open(char *);
-/* Closes a MIDI device */
-void midi_close(struct midi *);
+/* Gets the number of available ALSA MIDI devices */
+unsigned int midi_get_num_alsa_devices();
+/* Returns MIDI data structures for available ALSA MIDI devices */
+struct midi **midi_get_alsa_devices();
+/* Allocates a MIDI interface */
+struct midi *midi_alloc(unsigned int, char *, char *);
+/* Frees a MIDI structure */
+void midi_free(struct midi *);
+/* Opens the MIDI interfaces defined in a string */
+void midi_open_defined(struct midi **, unsigned int, char *, char *);
+/* Opens a MIDI interface */
+unsigned int midi_open(struct midi *, unsigned int);
+/* Closes a MIDI interface */
+void midi_close(struct midi *, unsigned int);
+/* Sets the current tick */
+void midi_set_tick(struct midi *, unsigned int);
 /* Stops a note playing on a MIDI channel using requested velocity */
 void midi_note_off(struct midi *, unsigned char, unsigned char, unsigned char);
 /* Plays a note on a MIDI channel using requested velocity */
@@ -60,13 +120,19 @@ void midi_channel_pressure(struct midi *, unsigned char, unsigned char);
 /* Sets the pitch wheel value of a MIDI channel */
 void midi_pitch_wheel(struct midi *, unsigned char, unsigned short);
 /* Sends a MIDI message */
-void midi_write(struct midi *, unsigned char *, unsigned short);
+void midi_write_raw(struct midi *, unsigned char *, unsigned short);
 /* Send a clock message */
 void midi_clock(struct midi *);
+/* Set the tempo (used when exporting) */
+void midi_tempo(struct midi *, unsigned int);
 /* Receives a MIDI message */
-unsigned char *midi_read(struct midi *);
+unsigned char *midi_read_raw(struct midi *);
 /* Receives a MIDI message */
 void midi_read_system_exclusive(struct midi *, struct message *, int);
+/* Returns the data written to the MIDI buffer */
+unsigned char *midi_get_buffer(struct midi *);
+/* Returns the length of data written to the MIDI buffer */
+int midi_get_buffer_length(struct midi *);
 /* Sets the length of a MIDI message */
 void message_set_length(struct message *, unsigned int);
 /* Sets the auto send flag of a MIDI message */
