@@ -1,6 +1,6 @@
 /* block.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -21,6 +21,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include "tutka.h"
 #include "block.h"
 
 /* Allocates a block */
@@ -41,10 +42,7 @@ struct block *block_alloc(int tracks, int length, int commandpages)
 /* Frees a block */
 void block_free(struct block *block)
 {
-  if (block == NULL) {
-    fprintf(stderr, "block_free() called with null block\n");
-    return;
-  }
+  nullcheck_void(block, block_free);
 
   if (block->name)
     free(block->name);
@@ -61,10 +59,7 @@ void block_free(struct block *block)
 /* Sets a note in a block */
 void block_set_note(struct block *block, unsigned int line, unsigned int track, unsigned char octave, unsigned char note, unsigned char instrument)
 {
-  if (block == NULL) {
-    fprintf(stderr, "block_set_note() called with null block\n");
-    return;
-  }
+  nullcheck_void(block, block_set_note);
 
   if (note) {
     block->notes[2 * (block->tracks * line + track)] = octave * 12 + note;
@@ -78,10 +73,7 @@ void block_set_note(struct block *block, unsigned int line, unsigned int track,
 /* Sets an instrument in a block */
 void block_set_instrument(struct block *block, unsigned int line, unsigned int track, unsigned char instrument)
 {
-  if (block == NULL) {
-    fprintf(stderr, "block_set_instrument() called with null block\n");
-    return;
-  }
+  nullcheck_void(block, block_set_instrument);
 
   block->notes[2 * (block->tracks * line + track) + 1] = instrument;
 }
@@ -89,10 +81,7 @@ void block_set_instrument(struct block *block, unsigned int line, unsigned int t
 /* Sets a part of a command in a block */
 void block_set_command(struct block *block, unsigned int line, unsigned int track, unsigned int commandpage, unsigned char slot, unsigned char data)
 {
-  if (block == NULL) {
-    fprintf(stderr, "block_set_command() called with null block\n");
-    return;
-  }
+  nullcheck_void(block, block_set_command);
 
   if (slot & 1) {
     block->commands[commandpage * 2 * block->tracks * block->length + 2 * (block->tracks * line + track) + slot / 2] &= 0xf0;
@@ -106,10 +95,7 @@ void block_set_command(struct block *block, unsigned int line, unsigned int trac
 /* Sets a command in a block */
 void block_set_command_full(struct block *block, unsigned int line, unsigned int track, unsigned int commandpage, unsigned char command, unsigned char data)
 {
-  if (block == NULL) {
-    fprintf(stderr, "block_set_command_full() called with null block\n");
-    return;
-  }
+  nullcheck_void(block, block_set_command_full);
 
   block->commands[commandpage * 2 * block->tracks * block->length + 2 * (block->tracks * line + track)] = command;
   block->commands[commandpage * 2 * block->tracks * block->length + 2 * (block->tracks * line + track) + 1] = data;
@@ -118,15 +104,12 @@ void block_set_command_full(struct block *block, unsigned int line, unsigned int
 /* Sets the number of tracks in a block */
 void block_set_tracks(struct block *block, unsigned int tracks)
 {
+  nullcheck_void(block, block_set_tracks);
+
   unsigned char *notes, *commands;
   unsigned int existingtracks;
   int l, t, ep, nl, nt;
 
-  if (block == NULL) {
-    fprintf(stderr, "block_set_tracks() called with null block\n");
-    return;
-  }
-
   /* Allocate new arrays */
   notes = (unsigned char *)calloc(2 * tracks * block->length, sizeof(unsigned char));
   commands = (unsigned char *)calloc(block->commandpages * 2 * tracks * block->length, sizeof(unsigned char));
@@ -165,15 +148,12 @@ void block_set_tracks(struct block *block, unsigned int tracks)
 /* Sets the length of a block */
 void block_set_length(struct block *block, unsigned int length)
 {
+  nullcheck_void(block, block_set_length);
+
   unsigned char *notes, *commands;
   unsigned int existinglength;
   int l, t, ep, nl, nt;
 
-  if (block == NULL) {
-    fprintf(stderr, "block_set_length() called with null block\n");
-    return;
-  }
-
   /* Allocate new arrays */
   notes = (unsigned char *)calloc(2 * block->tracks * length, sizeof(unsigned char));
   commands = (unsigned char *)calloc(block->commandpages * 2 * block->tracks * length, sizeof(unsigned char));
@@ -212,14 +192,11 @@ void block_set_length(struct block *block, unsigned int length)
 /* Sets the number of command pages in a block */
 void block_set_commandpages(struct block *block, unsigned int commandpages)
 {
+  nullcheck_void(block, block_set_commandpages);
+
   unsigned char *commands;
   unsigned int existingcommandpages;
 
-  if (block == NULL) {
-    fprintf(stderr, "block_set_commandpages() called with null block\n");
-    return;
-  }
-
   /* Allocate new command page array */
   commands = (unsigned char *)calloc(commandpages * 2 * block->tracks * block->length, sizeof(unsigned char));
 
@@ -241,17 +218,25 @@ void block_set_commandpages(struct block *block, unsigned int commandpages)
   block->commandpages = commandpages;
 }
 
+/* Sets the name of the block */
+void block_set_name(struct block *block, char *name)
+{
+  nullcheck_void(block, block_set_name);
+
+  if (block->name)
+    free(block->name);
+
+  block->name = (char *)strdup(name);  
+}
+
 /* Copies a part of a block to a new block */
 struct block *block_copy(struct block *block, int starttrack, int startline, int endtrack, int endline)
 {
+  nullcheck_pointer(block, block_copy);
+
   struct block *newblock;
   int t, l, ep, onl, nnl, ont, nnt;
 
-  if (block == NULL) {
-    fprintf(stderr, "block_copy() called with null block\n");
-    return NULL;
-  }
-
   /* Bounds checking */
   if (starttrack < 0)
     starttrack = 0;
@@ -307,12 +292,10 @@ struct block *block_copy(struct block *block, int starttrack, int startline, int
 /* Pastes a block to another block in the given position */
 void block_paste(struct block *block, struct block *from, int track, int line)
 {
-  int t, l, ep, nl, nt, fnl, fnt, cnl, cnt;
+  nullcheck_void(block, block_paste);
+  nullcheck_void(from, block_paste);
 
-  if (block == NULL || from == NULL) {
-    fprintf(stderr, "block_paste() called with null block\n");
-    return;
-  }
+  int t, l, ep, nl, nt, fnl, fnt, cnl, cnt;
 
   nl = block->length;
   nt = block->tracks;
@@ -342,12 +325,9 @@ void block_paste(struct block *block, struct block *from, int track, int line)
 /* Clears a part of a block */
 void block_clear(struct block *block, int starttrack, int startline, int endtrack, int endline)
 {
-  int t, l, ep, nl, nt;
+  nullcheck_void(block, block_clear);
 
-  if (block == NULL) {
-    fprintf(stderr, "block_clear() called with null block\n");
-    return;
-  }
+  int t, l, ep, nl, nt;
 
   /* Bounds checking */
   if (starttrack < 0)
@@ -479,15 +459,12 @@ struct block *block_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 /* Saves a block to an XML document */
 void block_save(struct block *block, int number, xmlNodePtr parent)
 {
+  nullcheck_void(block, block_save);
+
   char c[10];
   int i, j, k;
   xmlNodePtr node, subnode;
 
-  if (block == NULL) {
-    fprintf(stderr, "block_save() called with null block\n");
-    return;
-  }
-
   /* Set block properties */
   node = xmlNewChild(parent, NULL, "block", NULL);
   snprintf(c, 10, "%d", number);
@@ -540,12 +517,9 @@ void block_save(struct block *block, int number, xmlNodePtr parent)
 /* Transposes a block or a part of it */
 void block_transpose(struct block *block, int instrument, int halfnotes, int starttrack, int startline, int endtrack, int endline)
 {
-  int l, t;
+  nullcheck_void(block, block_transpose);
 
-  if (block == NULL) {
-    fprintf(stderr, "block_transpose() called with null block\n");
-    return;
-  }
+  int l, t;
 
   /* Bounds checking */
   if (starttrack < 0)
@@ -606,12 +580,9 @@ void block_transpose(struct block *block, int instrument, int halfnotes, int sta
 /* Expands/shrinks a block or a part of it */
 void block_expandshrink(struct block *block, int factor, int starttrack, int startline, int endtrack, int endline)
 {
-  int l, t, p;
+  nullcheck_void(block, block_expandshrink);
 
-  if (block == NULL) {
-    fprintf(stderr, "block_expandshrink() called with null block\n");
-    return;
-  }
+  int l, t, p;
 
   if (factor > -2 && factor < 2)
     return;
@@ -696,12 +667,9 @@ void block_expandshrink(struct block *block, int factor, int starttrack, int sta
 /* Changes an instrument to another */
 void block_changeinstrument(struct block *block, int from, int to, int swap, int starttrack, int startline, int endtrack, int endline)
 {
-  int l, t;
+  nullcheck_void(block, block_changeinstrument);
 
-  if (block == NULL) {
-    fprintf(stderr, "block_changeinstrument() called with null block\n");
-    return;
-  }
+  int l, t;
 
   /* Bounds checking */
   if (starttrack < 0)
