@@ -1,6 +1,6 @@
 /* gui.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -82,6 +82,8 @@ struct gui *gui_open(struct editor *editor)
   GtkTreeSelection *select;
   GtkListStore *store;
   GdkPixbuf *pixbuf;
+  GtkWidget *w;
+  char *songpath, *messagepath;
 
   gui->editor = editor;
 
@@ -209,18 +211,58 @@ struct gui *gui_open(struct editor *editor)
   g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_messagelist_toggled), gui);
   gtk_tree_view_append_column(treeview, new_column("Automatically Send Message After Loading Song", renderer, "active", GUI_MESSAGELIST_COLUMN_AUTOSEND, NULL));
 
+  /* Preferences */
+  store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
+  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
+  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+  gtk_tree_selection_set_mode(select, GTK_SELECTION_NONE);
+  gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME));
+  gtk_tree_view_append_column(treeview, new_column("Device", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, NULL));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME));
+  gtk_tree_view_append_column(treeview, new_column("Name", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, NULL));
+  renderer = gtk_cell_renderer_toggle_new();
+  g_object_set(renderer, "activatable", TRUE, NULL);
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED));
+  g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
+  gtk_tree_view_append_column(treeview, new_column("Input", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED, NULL));
+  renderer = gtk_cell_renderer_toggle_new();
+  g_object_set(renderer, "activatable", TRUE, NULL);
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED));
+  g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
+  gtk_tree_view_append_column(treeview, new_column("Output", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED, NULL));
+
   /* Set the keyboard octave */
-  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "optionmenu_keyboard")), 3);
   gui->editor->octave = 3;
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_keyboard")), gui->editor->octave);
+
+  /* Set default choices in combo boxes */
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_transpose_area")), 0);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_transpose_instrument")), 0);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_transpose_mode")), 0);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_changeinstrument_area")), 0);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_expandshrink_area")), 0);
+
+  /* Create the MIDI interface selector (libglade doesn't do this if no items are defined) */
+  w = glade_xml_get_widget(gui->xml, "combobox_instrumentproperties_midiinterface");
+  store = gtk_list_store_new(1, G_TYPE_STRING);
+  gtk_combo_box_set_model(GTK_COMBO_BOX(w), GTK_TREE_MODEL(store));
+  renderer = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
+  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(w), renderer, "text", 0, NULL);
 
   /* Set default path in file dialogs */
-  if (gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL) != NULL) {
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_file_open")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL));
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_file_saveas")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL));
+  songpath = editor_song_path_get(gui->editor);
+  if (songpath != NULL) {
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_open")), songpath);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_saveas")), songpath);
   }
-  if (gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL) != NULL) {
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_message_load")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL));
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_message_save")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL));
+  messagepath = editor_message_path_get(gui->editor);
+  if (messagepath != NULL) {
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_message_load")), messagepath);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_message_save")), messagepath);
   }
 
   /* Add icons */
@@ -239,6 +281,22 @@ void gui_close(struct gui *gui)
   free(gui);
 }
 
+/* Sets the current filename to the save dialog */
+void gui_set_saveas_filename(struct gui *gui, char *filename)
+{
+  /* Set default path in save as dialog */
+  if (filename != NULL) {
+    gchar *path, *name;
+
+    path = g_path_get_dirname(filename);
+    name = g_path_get_basename(filename);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_saveas")), path);
+    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_saveas")), name);
+    g_free(path);
+    g_free(name);
+  }
+}
+
 /* Refreshes all parts of the GUI */
 void gui_refresh_all(struct gui *gui)
 {
@@ -246,8 +304,8 @@ void gui_refresh_all(struct gui *gui)
   Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
 
   tracker_reset(tracker);
-  tracker_set_pattern(tracker, gui->editor->song->blocks[gui->editor->block]);
-  tracker_set_num_channels(tracker, gui->editor->song->blocks[gui->editor->block]->tracks);
+  tracker_set_pattern(tracker, editor_song_block_current_get(gui->editor));
+  tracker_set_num_channels(tracker, editor_song_block_current_tracks_get(gui->editor));
 
   /* Refresh the GUI */
   gui_info_refresh(gui);
@@ -270,6 +328,7 @@ void gui_refresh_all(struct gui *gui)
   tracker->cursor_ch = 0;
 
   /* The current CellRendererSpin implementation needs a pointer to the upper limit values */
+  /* FIX: this is not the way this should be done */
   if (gui->editor->song == NULL) {
     gui->numplayseqs = NULL;
     gui->numblocks = NULL;
@@ -429,16 +488,18 @@ void gui_refresh(struct gui *gui)
 void gui_tracker_refresh(struct gui *gui)
 {
   Tracker *tracker = gui_get_tracker(gui);
+  unsigned int length = editor_song_block_current_length_get(gui->editor);
+  unsigned int cmdpages = editor_song_block_current_commandpages_get(gui->editor);
 
   /* Check that the line is valid */
-  if (gui->editor->line >= gui->editor->song->blocks[gui->editor->block]->length)
-    gui->editor->line = gui->editor->song->blocks[gui->editor->block]->length - 1;
-  /* Check that the commad page is valid */
-  if (gui->editor->cmdpage > gui->editor->song->blocks[gui->editor->block]->commandpages)
-    gui->editor->cmdpage = gui->editor->song->blocks[gui->editor->block]->commandpages - 1;
+  if (gui->editor->line >= length)
+    gui->editor->line = length - 1;
+  /* Check that the command page is valid */
+  if (gui->editor->cmdpage > cmdpages)
+    gui->editor->cmdpage = cmdpages - 1;
 
-  tracker_set_num_channels(tracker, gui->editor->song->blocks[gui->editor->block]->tracks);
-  tracker_set_pattern(tracker, gui->editor->song->blocks[gui->editor->block]);
+  tracker_set_num_channels(tracker, editor_song_block_current_tracks_get(gui->editor));
+  tracker_set_pattern(tracker, editor_song_block_current_get(gui->editor));
   tracker_set_patpos(tracker, gui->editor->line);
   tracker_set_cmdpage(tracker, gui->editor->cmdpage);
 }
@@ -448,18 +509,20 @@ void gui_song_refresh(struct gui *gui)
 {
   GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_songname"));
   GtkToggleButton *togglebutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_sendsync"));
+  char *name;
 
   g_signal_handlers_block_by_func(entry, on_entry_songname_changed, gui);
   g_signal_handlers_block_by_func(togglebutton, on_checkbutton_sendsync_toggled, gui);
 
+  name = editor_song_name_get(gui->editor);
   /* Refresh the song name widget */
-  if (gui->editor->song->name != NULL)
-    gtk_entry_set_text(entry, gui->editor->song->name);
+  if (name != NULL)
+    gtk_entry_set_text(entry, name);
   else
     gtk_entry_set_text(entry, "");
 
   /* Refresh the send sync widget */
-  if (gui->editor->song->sendsync)
+  if (editor_song_sendsync_get(gui->editor))
     gtk_toggle_button_set_active(togglebutton, TRUE);
   else
     gtk_toggle_button_set_active(togglebutton, FALSE);
@@ -475,39 +538,52 @@ void gui_instrument_refresh(struct gui *gui)
   snprintf(instrument, 3, "%.2d", gui->editor->instrument + 1);
   GtkSpinButton *spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_instrument"));
   GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrumentproperties_name"));
+  GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_instrumentproperties_midiinterface"));
   GtkRange *range_volume = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_volume"));
   GtkRange *range_transpose = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_transpose"));
   GtkRange *range_hold = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_hold"));
   GtkRange *range_midich = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_midich"));
+  char *name;
+  int i;
 
   /* Refresh the main window widgets */
   g_signal_handlers_block_by_func(spinbutton, on_spinbutton_instrument_changed, gui);
   g_signal_handlers_block_by_func(entry, on_entry_instrumentproperties_name_changed, gui);
+  g_signal_handlers_block_by_func(combobox, on_combobox_instrumentproperties_midiinterface_changed, gui);
   g_signal_handlers_block_by_func(range_volume, on_scale_volume_value_changed, gui);
   g_signal_handlers_block_by_func(range_transpose, on_scale_transpose_value_changed, gui);
   g_signal_handlers_block_by_func(range_hold, on_scale_hold_value_changed, gui);
   g_signal_handlers_block_by_func(range_midich, on_scale_midich_value_changed, gui);
-
   gtk_spin_button_set_value(spinbutton, gui->editor->instrument + 1);
 
-  if (gui->editor->song->instruments[gui->editor->instrument]->name != NULL)
-    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrument")), gui->editor->song->instruments[gui->editor->instrument]->name);
+  name = editor_song_instrument_current_name_get(gui->editor);
+  if (name != NULL)
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrument")), name);
   else
     gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrument")), "");
 
   /* Refresh the instrument window widgets */
-  if (gui->editor->song->instruments[gui->editor->instrument]->name != NULL)
-    gtk_entry_set_text(entry, gui->editor->song->instruments[gui->editor->instrument]->name);
+  if (name != NULL)
+    gtk_entry_set_text(entry, name);
   else
     gtk_entry_set_text(entry, "");
 
-  gtk_adjustment_set_value(gtk_range_get_adjustment(range_volume), gui->editor->song->instruments[gui->editor->instrument]->defaultvelocity);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(range_transpose), gui->editor->song->instruments[gui->editor->instrument]->transpose);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(range_hold), gui->editor->song->instruments[gui->editor->instrument]->hold);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(range_midich), gui->editor->song->instruments[gui->editor->instrument]->midichannel + 1);
+  GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(combobox));
+  gtk_list_store_clear(store);
+  for (i = 0; i < gui->editor->nummidiinterfaces; i++) {
+    GtkTreeIter iter;
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter, 0, gui->editor->midiinterfaces[i]->name, -1);
+  }
+  gtk_combo_box_set_active(combobox, editor_song_instrument_current_midiinterface_get(gui->editor));
+  gtk_adjustment_set_value(gtk_range_get_adjustment(range_volume), editor_song_instrument_current_defaultvelocity_get(gui->editor));
+  gtk_adjustment_set_value(gtk_range_get_adjustment(range_transpose), editor_song_instrument_current_transpose_get(gui->editor));
+  gtk_adjustment_set_value(gtk_range_get_adjustment(range_hold), editor_song_instrument_current_hold_get(gui->editor));
+  gtk_adjustment_set_value(gtk_range_get_adjustment(range_midich), editor_song_instrument_current_midichannel_get(gui->editor) + 1);
 
   g_signal_handlers_unblock_by_func(spinbutton, on_spinbutton_instrument_changed, gui);
   g_signal_handlers_unblock_by_func(entry, on_entry_instrumentproperties_name_changed, gui);
+  g_signal_handlers_unblock_by_func(combobox, on_combobox_instrumentproperties_midiinterface_changed, gui);
   g_signal_handlers_unblock_by_func(range_volume, on_scale_volume_value_changed, gui);
   g_signal_handlers_unblock_by_func(range_transpose, on_scale_transpose_value_changed, gui);
   g_signal_handlers_unblock_by_func(range_hold, on_scale_hold_value_changed, gui);
@@ -524,8 +600,8 @@ void gui_tempo_refresh(struct gui *gui)
   g_signal_handlers_block_by_func(scale_tempo, on_scale_tempo_value_changed, gui);
   g_signal_handlers_block_by_func(scale_tpl, on_scale_tpl_value_changed, gui);
 
-  gtk_adjustment_set_value(gtk_range_get_adjustment(scale_tempo), gui->editor->song->tempo);
-  gtk_adjustment_set_value(gtk_range_get_adjustment(scale_tpl), gui->editor->song->ticksperline);
+  gtk_adjustment_set_value(gtk_range_get_adjustment(scale_tempo), editor_song_tempo_get(gui->editor));
+  gtk_adjustment_set_value(gtk_range_get_adjustment(scale_tpl), editor_song_ticksperline_get(gui->editor));
 
   g_signal_handlers_unblock_by_func(scale_tempo, on_scale_tempo_value_changed, gui);
   g_signal_handlers_unblock_by_func(scale_tpl, on_scale_tpl_value_changed, gui);
@@ -543,7 +619,7 @@ void gui_statusbar_refresh(struct gui *gui)
   GtkLabel *status = GTK_LABEL(glade_xml_get_widget(gui->xml, "label_status"));
 
   /* Status bar */
-  switch (gui->editor->player->mode) {
+  switch (editor_player_mode_get(gui->editor)) {
   case MODE_PLAY_SONG:
     gtk_label_set_text(status, "Playing song");
     break;
@@ -561,21 +637,21 @@ void gui_info_refresh(struct gui *gui)
 {
   char *section, *playseq, *position, *block, *cmdpage;
   /* Calculate lengths of the info strings and allocate memory for them */
-  int sectionlength = 8 + intlen(gui->editor->section + 1) + 1 + intlen(gui->editor->song->numsections) + 1;
+  int sectionlength = 8 + intlen(gui->editor->section + 1) + 1 + intlen(editor_song_numsections_get(gui->editor)) + 1;
   int playseqlength;
-  int positionlength = 9 + intlen(gui->editor->position + 1) + 1 + intlen(gui->editor->song->playseqs[gui->editor->playseq]->length) + 1;
+  int positionlength = 9 + intlen(gui->editor->position + 1) + 1 + intlen(editor_song_playseq_current_length_get(gui->editor)) + 1;
   int blocklength;
-  int cmdpagelength = 13 + intlen(gui->editor->cmdpage + 1) + 1 + intlen(gui->editor->song->blocks[gui->editor->block]->commandpages) + 1;
+  int cmdpagelength = 13 + intlen(gui->editor->cmdpage + 1) + 1 + intlen(editor_song_block_current_commandpages_get(gui->editor)) + 1;
 
-  if (gui->editor->song->playseqs[gui->editor->playseq]->name != NULL)
-    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(gui->editor->song->numplayseqs) + 2 + strlen(gui->editor->song->playseqs[gui->editor->playseq]->name) + 1;
+  if (editor_song_playseq_current_name_get(gui->editor) != NULL)
+    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(editor_song_numplayseqs_get(gui->editor)) + 2 + strlen(editor_song_playseq_current_name_get(gui->editor)) + 1;
   else
-    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(gui->editor->song->numplayseqs) + 1;
+    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(editor_song_numplayseqs_get(gui->editor)) + 1;
 
-  if (gui->editor->song->blocks[gui->editor->block]->name != NULL)
-    blocklength = 6 + intlen(gui->editor->block + 1) + 1 + intlen(gui->editor->song->numblocks) + 2 + strlen(gui->editor->song->blocks[gui->editor->block]->name) + 1;
+  if (editor_song_block_current_name_get(gui->editor) != NULL)
+    blocklength = 6 + intlen(gui->editor->block + 1) + 1 + intlen(editor_song_numblocks_get(gui->editor)) + 2 + strlen(editor_song_block_current_name_get(gui->editor)) + 1;
   else
-    blocklength = 6 + intlen(gui->editor->block + 1) + 1 + intlen(gui->editor->song->numblocks) + 2;
+    blocklength = 6 + intlen(gui->editor->block + 1) + 1 + intlen(editor_song_numblocks_get(gui->editor)) + 2;
 
   section = malloc(sectionlength);
   playseq = malloc(playseqlength);
@@ -584,17 +660,17 @@ void gui_info_refresh(struct gui *gui)
   cmdpage = malloc(cmdpagelength);
 
   /* Print the info strings */
-  snprintf(section, sectionlength, "Section %d/%d", gui->editor->section + 1, gui->editor->song->numsections);
-  if (gui->editor->song->playseqs[gui->editor->playseq]->name != NULL)
-    snprintf(playseq, playseqlength, "Playing Sequence %d/%d: %s", gui->editor->playseq + 1, gui->editor->song->numplayseqs, gui->editor->song->playseqs[gui->editor->playseq]->name);
+  snprintf(section, sectionlength, "Section %d/%d", gui->editor->section + 1, editor_song_numsections_get(gui->editor));
+  if (editor_song_playseq_current_name_get(gui->editor) != NULL)
+    snprintf(playseq, playseqlength, "Playing Sequence %d/%d: %s", gui->editor->playseq + 1, editor_song_numplayseqs_get(gui->editor), editor_song_playseq_current_name_get(gui->editor));
   else
-    snprintf(playseq, playseqlength, "Playing Sequence %d/%d", gui->editor->playseq + 1, gui->editor->song->numplayseqs);
-  snprintf(position, playseqlength, "Position %d/%d", gui->editor->position + 1, gui->editor->song->playseqs[gui->editor->playseq]->length);
-  if (gui->editor->song->blocks[gui->editor->block]->name != NULL)
-    snprintf(block, blocklength, "Block %d/%d: %s", gui->editor->block + 1, gui->editor->song->numblocks, gui->editor->song->blocks[gui->editor->block]->name);
+    snprintf(playseq, playseqlength, "Playing Sequence %d/%d", gui->editor->playseq + 1, editor_song_numplayseqs_get(gui->editor));
+  snprintf(position, playseqlength, "Position %d/%d", gui->editor->position + 1, editor_song_playseq_current_length_get(gui->editor));
+  if (editor_song_block_current_name_get(gui->editor) != NULL)
+    snprintf(block, blocklength, "Block %d/%d: %s", gui->editor->block + 1, editor_song_numblocks_get(gui->editor), editor_song_block_current_name_get(gui->editor));
   else
-    snprintf(block, blocklength, "Block %d/%d", gui->editor->block + 1, gui->editor->song->numblocks);
-  snprintf(cmdpage, cmdpagelength, "Command Page %d/%d", gui->editor->cmdpage + 1, gui->editor->song->blocks[gui->editor->block]->commandpages);
+    snprintf(block, blocklength, "Block %d/%d", gui->editor->block + 1, editor_song_numblocks_get(gui->editor));
+  snprintf(cmdpage, cmdpagelength, "Command Page %d/%d", gui->editor->cmdpage + 1, editor_song_block_current_commandpages_get(gui->editor));
   /* Refresh the info string widgets */
   gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_section")), section);
   gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_playseq")), playseq);
@@ -607,6 +683,7 @@ void gui_info_refresh(struct gui *gui)
   free(playseq);
   free(position);
   free(block);
+  free(cmdpage);
 }
 
 /* Refresh the timer */
@@ -633,14 +710,16 @@ void gui_blocklist_refresh(struct gui *gui, gboolean recreate)
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_blocklist"));
 
   if (recreate) {
+    int n = editor_song_numblocks_get(gui->editor);
+
     /* Recreate the block list */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
     g_signal_handlers_block_by_func(treeview, on_selection_blocklist_changed, gui);
     gtk_list_store_clear(store);
-    for (i = 0; i < gui->editor->song->numblocks; i++) {
+    for (i = 0; i < n; i++) {
       GtkTreeIter iter;
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, GUI_BLOCKLIST_COLUMN_NUMBER, i + 1, GUI_BLOCKLIST_COLUMN_NAME, gui->editor->song->blocks[i]->name, GUI_BLOCKLIST_COLUMN_TRACKS, gui->editor->song->blocks[i]->tracks, GUI_BLOCKLIST_COLUMN_LENGTH, gui->editor->song->blocks[i]->length, GUI_BLOCKLIST_COLUMN_COMMAND_PAGES, gui->editor->song->blocks[i]->commandpages, -1);
+      gtk_list_store_set(store, &iter, GUI_BLOCKLIST_COLUMN_NUMBER, i + 1, GUI_BLOCKLIST_COLUMN_NAME, editor_song_block_name_get(gui->editor, i), GUI_BLOCKLIST_COLUMN_TRACKS, editor_song_block_tracks_get(gui->editor, i), GUI_BLOCKLIST_COLUMN_LENGTH, editor_song_block_length_get(gui->editor, i), GUI_BLOCKLIST_COLUMN_COMMAND_PAGES, editor_song_block_commandpages_get(gui->editor, i), -1);
     }
     g_signal_handlers_unblock_by_func(treeview, on_selection_blocklist_changed, gui);
   }
@@ -652,7 +731,7 @@ void gui_blocklist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
 
   /* Make the delete button sensitive/insensitive */
-  if (gui->editor->song->numblocks > 1)
+  if (editor_song_numblocks_get(gui->editor) > 1)
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_blocklist_delete"), TRUE);
   else
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_blocklist_delete"), FALSE);
@@ -670,14 +749,16 @@ void gui_playseqlist_refresh(struct gui *gui, gboolean recreate)
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_playseqlist"));
 
   if (recreate) {
+    int n = editor_song_numplayseqs_get(gui->editor);
+
     /* Recreate the playing sequence list */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
     g_signal_handlers_block_by_func(treeview, on_selection_playseqlist_changed, gui);
     gtk_list_store_clear(store);
-    for (i = 0; i < gui->editor->song->numplayseqs; i++) {
+    for (i = 0; i < n; i++) {
       GtkTreeIter iter;
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, GUI_PLAYSEQLIST_COLUMN_NUMBER, i + 1, GUI_PLAYSEQLIST_COLUMN_NAME, gui->editor->song->playseqs[i]->name, -1);
+      gtk_list_store_set(store, &iter, GUI_PLAYSEQLIST_COLUMN_NUMBER, i + 1, GUI_PLAYSEQLIST_COLUMN_NAME, editor_song_playseq_name_get(gui->editor, i), -1);
     }
     g_signal_handlers_unblock_by_func(treeview, on_selection_playseqlist_changed, gui);
   }
@@ -689,7 +770,7 @@ void gui_playseqlist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
 
   /* Make the delete playing sequence button sensitive/insensitive */
-  if (gui->editor->song->numplayseqs > 1)
+  if (editor_song_numplayseqs_get(gui->editor) > 1)
     gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseqlist_delete")), TRUE);
   else
     gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseqlist_delete")), FALSE);
@@ -702,33 +783,37 @@ void gui_playseq_refresh(struct gui *gui, gboolean recreate)
   char playseqpos[10], *title;
   GtkTreeView *treeview;
   GtkTreePath *treepath;
+  char *name;
 
   snprintf(playseqpos, 10, "%d", gui->editor->position);
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_playseq"));
 
+  name = editor_song_playseq_current_name_get(gui->editor);
   /* Calculate length of the window title and allocate memory for it */
-  if (gui->editor->song->playseqs[gui->editor->playseq]->name != NULL)
-    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(gui->editor->song->numplayseqs) + 2 + strlen(gui->editor->song->playseqs[gui->editor->playseq]->name) + 1;
+  if (name != NULL)
+    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(editor_song_numplayseqs_get(gui->editor)) + 2 + strlen(name) + 1;
   else
-    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(gui->editor->song->numplayseqs) + 1;
+    playseqlength = 17 + intlen(gui->editor->playseq + 1) + 1 + intlen(editor_song_numplayseqs_get(gui->editor)) + 1;
 
   title = malloc(playseqlength);
 
   /* Print the window title */
-  if (gui->editor->song->playseqs[gui->editor->playseq]->name != NULL)
-    snprintf(title, playseqlength, "Playing Sequence %d/%d: %s", gui->editor->playseq + 1, gui->editor->song->numplayseqs, gui->editor->song->playseqs[gui->editor->playseq]->name);
+  if (name != NULL)
+    snprintf(title, playseqlength, "Playing Sequence %d/%d: %s", gui->editor->playseq + 1, editor_song_numplayseqs_get(gui->editor), name);
   else
-    snprintf(title, playseqlength, "Playing Sequence %d/%d", gui->editor->playseq + 1, gui->editor->song->numplayseqs);
+    snprintf(title, playseqlength, "Playing Sequence %d/%d", gui->editor->playseq + 1, editor_song_numplayseqs_get(gui->editor));
 
   if (recreate) {
+    int n = editor_song_playseq_current_length_get(gui->editor);
+
     /* Recreate the playing sequence */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
     g_signal_handlers_block_by_func(treeview, on_selection_playseq_changed, gui);
     gtk_list_store_clear(store);
-    for (i = 0; i < gui->editor->song->playseqs[gui->editor->playseq]->length; i++) {
+    for (i = 0; i < n; i++) {
       GtkTreeIter iter;
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, GUI_PLAYSEQ_COLUMN_POSITION, i + 1, GUI_PLAYSEQ_COLUMN_BLOCK_NUMBER, gui->editor->song->playseqs[gui->editor->playseq]->blocknumbers[i] + 1, GUI_PLAYSEQ_COLUMN_BLOCK_NAME, gui->editor->song->blocks[gui->editor->song->playseqs[gui->editor->playseq]->blocknumbers[i]]->name, -1);
+      gtk_list_store_set(store, &iter, GUI_PLAYSEQ_COLUMN_POSITION, i + 1, GUI_PLAYSEQ_COLUMN_BLOCK_NUMBER, editor_song_playseq_current_block_get(gui->editor, i) + 1, GUI_PLAYSEQ_COLUMN_BLOCK_NAME, editor_song_block_name_get(gui->editor, editor_song_playseq_current_block_get(gui->editor, i)), -1);
     }
     g_signal_handlers_unblock_by_func(treeview, on_selection_playseq_changed, gui);
   }
@@ -743,7 +828,7 @@ void gui_playseq_refresh(struct gui *gui, gboolean recreate)
   gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_playseq")), title);
 
   /* Make the delete position button sensitive/insensitive */
-  if (gui->editor->song->playseqs[gui->editor->playseq]->length > 1)
+  if (editor_song_playseq_current_length_get(gui->editor) > 1)
     gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_delete")), TRUE);
   else
     gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(gui->xml, "button_playseq_delete")), FALSE);
@@ -763,14 +848,16 @@ void gui_sectionlist_refresh(struct gui *gui, gboolean recreate)
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_sectionlist"));
 
   if (recreate) {
+    int n = editor_song_numsections_get(gui->editor);
+
     /* Recreate the section list */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
     g_signal_handlers_block_by_func(treeview, on_selection_sectionlist_changed, gui);
     gtk_list_store_clear(store);
-    for (i = 0; i < gui->editor->song->numsections; i++) {
+    for (i = 0; i < n; i++) {
       GtkTreeIter iter;
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, GUI_SECTIONLIST_COLUMN_SECTION, i + 1, GUI_SECTIONLIST_COLUMN_PLAYSEQ_NUMBER, gui->editor->song->sections[i] + 1, GUI_SECTIONLIST_COLUMN_PLAYSEQ_NAME, gui->editor->song->playseqs[gui->editor->song->sections[i]]->name, -1);
+      gtk_list_store_set(store, &iter, GUI_SECTIONLIST_COLUMN_SECTION, i + 1, GUI_SECTIONLIST_COLUMN_PLAYSEQ_NUMBER, editor_song_section_get(gui->editor, i) + 1, GUI_SECTIONLIST_COLUMN_PLAYSEQ_NAME, editor_song_playseq_name_get(gui->editor, editor_song_section_get(gui->editor, i)), -1);
     }
     g_signal_handlers_unblock_by_func(treeview, on_selection_sectionlist_changed, gui);
   }
@@ -782,7 +869,7 @@ void gui_sectionlist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
 
   /* Make the delete button sensitive/insensitive */
-  if (gui->editor->song->numsections > 1)
+  if (editor_song_numsections_get(gui->editor) > 1)
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_delete"), TRUE);
   else
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_sectionlist_delete"), FALSE);
@@ -795,21 +882,23 @@ void gui_messagelist_refresh(struct gui *gui, gboolean recreate)
   char oldmessage[10];
   GtkTreeView *treeview;
 
-  if (gui->editor->song->nummessages == 0)
+  if (editor_song_nummessages_get(gui->editor) == 0)
     gui->messagelist_message = -1;
 
   snprintf(oldmessage, 10, "%d", gui->messagelist_message);
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_messagelist"));
 
   if (recreate) {
+    int n = editor_song_nummessages_get(gui->editor);
+
     /* Recreate the Message list */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
     g_signal_handlers_block_by_func(treeview, on_selection_messagelist_changed, gui);
     gtk_list_store_clear(store);
-    for (i = 0; i < gui->editor->song->nummessages; i++) {
+    for (i = 0; i < n; i++) {
       GtkTreeIter iter;
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, GUI_MESSAGELIST_COLUMN_NUMBER, i, GUI_MESSAGELIST_COLUMN_NAME, gui->editor->song->messages[i]->name, GUI_MESSAGELIST_COLUMN_LENGTH, gui->editor->song->messages[i]->length, GUI_MESSAGELIST_COLUMN_AUTOSEND, gui->editor->song->messages[i]->autosend, -1);
+      gtk_list_store_set(store, &iter, GUI_MESSAGELIST_COLUMN_NUMBER, i, GUI_MESSAGELIST_COLUMN_NAME, editor_song_message_name_get(gui->editor, i), GUI_MESSAGELIST_COLUMN_LENGTH, editor_song_message_length_get(gui->editor, i), GUI_MESSAGELIST_COLUMN_AUTOSEND, editor_song_message_autosend_get(gui->editor, i), -1);
     }
     g_signal_handlers_unblock_by_func(treeview, on_selection_messagelist_changed, gui);
   }
@@ -836,25 +925,31 @@ void gui_trackvolumes_refresh(struct gui *gui)
 {
   /* Refresh the track volumes window widgets */
   int i;
-  char track[10];
   GtkWidget *table = glade_xml_get_widget(gui->xml, "table_trackvolumes");
-
+  int n = editor_song_maxtracks_get(gui->editor);
+  char track[10];
+  
   /* Destroy all children */
   gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback) gtk_widget_destroy, NULL);
-  gtk_table_resize(GTK_TABLE(table), 3, gui->editor->song->maxtracks);
+  gtk_table_resize(GTK_TABLE(table), 5, n);
 
   /* Create widgets for each channel */
-  for (i = 0; i < gui->editor->song->maxtracks; i++) {
+  for (i = 0; i < n; i++) {
+    unsigned int volume = editor_song_track_volume_get(gui->editor, i);
+    unsigned int mute = editor_song_track_mute_get(gui->editor, i);
+    unsigned int solo = editor_song_track_solo_get(gui->editor, i);
+    char *name = editor_song_track_name_get(gui->editor, i);
     GtkWidget *w, *x;
 
     /* Track numbers */
-    snprintf(track, 10, "Track %d", i + 1);
+    snprintf(track, 10, "Track %d", i);
     x = gtk_label_new(track);
     gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
-
+    
     /* Volume sliders */
-    w = (GtkWidget *) gtk_adjustment_new((gui->editor->song->trackvolumes[i]) & 127, 0, 127, 1, 8, 0);
-    g_signal_connect(G_OBJECT(w), "value_changed", G_CALLBACK(on_adjustment_trackvolumes_changed), (gpointer) & gui->editor->song->trackvolumes[i]);
+    w = (GtkWidget *) gtk_adjustment_new(volume, 0, 127, 1, 8, 0);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    g_signal_connect(G_OBJECT(w), "value_changed", G_CALLBACK(on_adjustment_trackvolumes_volume_changed), gui);
     x = gtk_vscale_new(GTK_ADJUSTMENT(w));
     gtk_range_set_inverted(GTK_RANGE(x), TRUE);
     gtk_scale_set_digits(GTK_SCALE(x), 0);
@@ -862,10 +957,30 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
     /* Mute buttons */
     w = gtk_check_button_new_with_label("Mute");
-    if (gui->editor->song->trackvolumes[i] > 127)
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    if (mute)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
-    g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_toggled), (gpointer) & gui->editor->song->trackvolumes[i]);
+    g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_mute_toggled), gui);
     gtk_table_attach(GTK_TABLE(table), w, i, i + 1, 2, 3, GTK_EXPAND, GTK_FILL, 2, 4);
+
+    /* Solo buttons */
+    w = gtk_check_button_new_with_label("Solo");
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    if (solo)
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+    g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_solo_toggled), gui);
+    gtk_table_attach(GTK_TABLE(table), w, i, i + 1, 3, 4, GTK_EXPAND, GTK_FILL, 2, 4);
+
+    /* Track names */
+    x = gtk_entry_new();
+    gtk_entry_set_width_chars(GTK_ENTRY(x), 7);
+    g_object_set_data(G_OBJECT(x), "track", (gpointer)i);
+    g_signal_connect(G_OBJECT(x), "changed", G_CALLBACK(on_entry_trackvolumes_name_changed), gui);
+    if (name != NULL)
+      gtk_entry_set_text(GTK_ENTRY(x), name);
+    else
+      gtk_entry_set_text(GTK_ENTRY(x), "");
+    gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
   }
 
   gtk_widget_show_all(table);
@@ -873,23 +988,40 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
 void gui_preferences_refresh(struct gui *gui)
 {
-  GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_preferences_midi_device"));
-  GtkMenuItem *menu_rtc = GTK_MENU_ITEM(glade_xml_get_widget(gui->xml, "menuitem_preferences_rtc"));
-  GtkMenuItem *menu_nanosleep = GTK_MENU_ITEM(glade_xml_get_widget(gui->xml, "menuitem_preferences_nanosleep"));
+  GtkTreeView *treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
+  GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_preferences_scheduling_mode"));
+  int i;
 
   /* Refresh the preferences window widgets */
-  g_signal_handlers_block_by_func(entry, on_entry_preferences_midi_device_changed, gui);
-  g_signal_handlers_block_by_func(menu_rtc, on_menuitem_preferences_rtc_activate, gui);
-  g_signal_handlers_block_by_func(menu_nanosleep, on_menuitem_preferences_nanosleep_activate, gui);
+  g_signal_handlers_block_by_func(treeview, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_block_by_func(combobox, on_combobox_preferences_scheduling_mode_changed, gui);
 
-  if (gconf_client_get_string(gui->editor->gconfclient, "/apps/tutka/mididevice", NULL) != NULL)
-    gtk_entry_set_text(entry, gconf_client_get_string(gui->editor->gconfclient, "/apps/tutka/mididevice", NULL));
-  else
-    gtk_entry_set_text(entry, "");
+  /* Recreate the Message list */
+  GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
+  gtk_list_store_clear(store);
+  for (i = 1; i < gui->editor->nummidiinterfaces; i++) {
+    GtkTreeIter iter;
+    char *device, *name;
+    gboolean inputenabled = FALSE, outputenabled = FALSE;
 
-  gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "optionmenu_preferences_scheduling_mode")), gconf_client_get_int(gui->editor->gconfclient, "/apps/tutka/scheduler", NULL));
+    if (gui->editor->midiinterfaces[i]->devicename != NULL)
+      device = gui->editor->midiinterfaces[i]->devicename;
+    else
+      device = "";
+    if (gui->editor->midiinterfaces[i]->name != NULL)
+      name = gui->editor->midiinterfaces[i]->name;
+    else
+      name = "";
+    if (gui->editor->midiinterfaces[i]->direction & DIRECTION_IN)
+      inputenabled = TRUE;
+    if (gui->editor->midiinterfaces[i]->direction & DIRECTION_OUT)
+      outputenabled = TRUE;
 
-  g_signal_handlers_unblock_by_func(entry, on_entry_preferences_midi_device_changed, gui);
-  g_signal_handlers_unblock_by_func(menu_rtc, on_menuitem_preferences_rtc_activate, gui);
-  g_signal_handlers_unblock_by_func(menu_nanosleep, on_menuitem_preferences_nanosleep_activate, gui);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, device, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED, inputenabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED, outputenabled, -1);
+  }
+
+  gtk_combo_box_set_active(combobox, editor_scheduler_get(gui->editor) - SCHED_RTC);
+  g_signal_handlers_unblock_by_func(treeview, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_unblock_by_func(combobox, on_combobox_preferences_scheduling_mode_changed, gui);
 }
