@@ -1,6 +1,6 @@
 /* main.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -32,10 +32,12 @@
 #include "conversion.h"
 
 extern gboolean gui_quit;
+char *nullcheck_error_string = "%s() called with null %s\n";
 
 int main(int argc, char *argv[])
 {
   struct editor *editor = NULL;
+  int i;
 
   if (!g_thread_supported()) {
     g_thread_init(NULL);
@@ -67,10 +69,11 @@ int main(int argc, char *argv[])
 	gtk_main_iteration_do(FALSE);
 
       /* Read MIDI input if there is any pending */
-      while ((data = midi_read(editor->player->midi)) != NULL) {
-	editor_midi_input(editor, data);
-	free(data);
-      }
+      for (i = 0; i < editor->nummidiinterfaces; i++)
+        while ((data = midi_read_raw(editor->midiinterfaces[i])) != NULL) {
+          editor_midi_input(editor, data);
+          free(data);
+        }
 
       /* Refresh the GUI */
       gui_refresh(editor->gui);
