@@ -1,6 +1,6 @@
 /* mmd.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "tutka.h"
 #include "mmd.h"
 
 #define getlong(a) (((*(a+0))<<24)|((*(a+1))<<16)|((*(a+2))<<8)|((*(a+3))))
@@ -33,12 +34,9 @@
 
 int MMD2_length_get(struct MMD2 *mmd)
 {
-  int length = 0, i, j;
+  nullcheck_int(mmd, MMD2_length_get);
 
-  if (mmd == NULL) {
-    fprintf(stderr, "MMD2_length_get() called with null mmd\n");
-    return 0;
-  }
+  int length = 0, i, j;
 
   /* MMD2 */
   length += sizeof(struct MMD2);
@@ -154,6 +152,8 @@ int MMD2_length_get(struct MMD2 *mmd)
 /* Loads an MMD2 file and returns a pointer to its MMD0 structure */
 struct MMD2 *MMD2_load(char *filename)
 {
+  nullcheck_pointer(filename, MMD2_load);
+
   FILE *file;
   struct MMD2 *mmd;
   int flength;
@@ -181,15 +181,13 @@ struct MMD2 *MMD2_load(char *filename)
 /* Saves an MMD2 file */
 void MMD2_save(struct MMD2 *mmd, char *filename)
 {
+  nullcheck_void(mmd, MMD2_save);
+  nullcheck_void(filename, MMD2_save);
+
   FILE *file;
   int offsets[16], i, j;
   unsigned char *data;
 
-  if (mmd == NULL) {
-    fprintf(stderr, "MMD2_save() called with null mmd\n");
-    return;
-  }
-
   if (mmd->modlen == 0)
     mmd->modlen = MMD2_length_get(mmd);
 
@@ -553,51 +551,48 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
 /* Frees an MMD0 structure and all associated data */
 void MMD2_free(struct MMD2 *mmd)
 {
+  nullcheck_void(mmd, MMD2_free);
+
   int i;
 
-  if (mmd) {
-    for (i = 0; i < mmd->song->numblocks - 1; i++) {
-      if (mmd->blockarr[i])
-	MMD1Block_free(mmd->blockarr[i]);
-    }
-    if (mmd->blockarr)
-      free(mmd->blockarr);
-
-    if (mmd->song)
-      MMD2song_free(mmd->song);
-
-    if (mmd->expdata)
-      free(mmd->expdata);
-
-    free(mmd);
+  for (i = 0; i < mmd->song->numblocks - 1; i++) {
+    if (mmd->blockarr[i])
+      MMD1Block_free(mmd->blockarr[i]);
   }
+  if (mmd->blockarr)
+    free(mmd->blockarr);
+
+  if (mmd->song)
+    MMD2song_free(mmd->song);
+
+  if (mmd->expdata)
+    free(mmd->expdata);
+
+  free(mmd);
 }
 
 /* Frees an MMD2song structure and all associated data */
 void MMD2song_free(struct MMD2song *song)
 {
+  nullcheck_void(song, MMD2song_free);
+
   unsigned short i;
 
-  if (song) {
-    for (i = 0; i < song->numpseqs; i++) {
-      if (song->playseqtable[i]) {
-	free(song->playseqtable[i]);
-	song->playseqtable[i] = 0;
-      }
+  for (i = 0; i < song->numpseqs; i++) {
+    if (song->playseqtable[i]) {
+      free(song->playseqtable[i]);
+      song->playseqtable[i] = 0;
     }
-    free(song);
   }
+  free(song);
 }
 
 /* Frees an MMD1Block structure and all associated data */
 void MMD1Block_free(struct MMD1Block *block)
 {
-  int i;
+  nullcheck_void(block, MMD1block_free);
 
-  if (block == NULL) {
-    fprintf(stderr, "MMD1Block_free() called with null block\n");
-    return;
-  }
+  int i;
 
   if (block->info) {
     if (block->info->pagetable) {
@@ -614,8 +609,9 @@ void MMD1Block_free(struct MMD1Block *block)
 
 void PlaySeq_free(struct PlaySeq *playseq)
 {
-  if (playseq)
-    free(playseq);
+  nullcheck_void(playseq, PlaySeq_free);
+
+  free(playseq);
 }
 
 struct MMD2 *MMD2_parse(unsigned char *base, int offset)
