@@ -1,6 +1,6 @@
 /* tutkakillallnotes.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -27,20 +27,23 @@
 #include <unistd.h>
 #include <glib.h>
 #include "midi.h"
+char *nullcheck_error_string = "%s() called with null %s\n";
 
-int main(int argc, char *argv[]) {
+int main(int argc, char *argv[])
+{
   struct midi *midi;
   int i, j;
 
-  midi=midi_open("/dev/midi");
-
-  for(i=0; i<16; i++)
-    for(j=0; j<128; j++)
-      midi_note_off(midi, i, j, 127); 
+  midi = midi_alloc(MIDI_OSS, "/dev/midi", NULL);
+  midi_open(midi, DIRECTION_OUT);
+  
+  for (i = 0; i < 16; i++)
+    for (j = 0; j < 128; j++)
+      midi_note_off(midi, i, j, 127);
 
   sleep(3);
 
-  midi_close(midi);
+  midi_free(midi);
 
   return 0;
 }
